(* ::Package:: *)

(* mcirc.m*)

(* Version 3 *)
 
(*Created by Rolf Sulanke, Sep 8, 1998, last revision Nov 24, 2011.*)   
  
(*The package contains definitions and procedures for 
the Moebius geometry of circles in the 3-sphere. It continues the package \
euvec.m which contains concepts of the Euclidean geometry of spheres and circles. 
Operations and functions of pseudo-Euclidean vector algebra (see 
the package neuvec.m and the notebook pseuklid.m) are used. The package is
tested, explained, and  applied in the notebooks emg2.nb, titled Elementaary 
Moebius Geometry II.*)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Elementary Moebius Geometry". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)

(*Changes July 6,2000: circle2spv, plotcircle2spv new definitions!
Nov. 2011: The Euclidean functions and modules havebeen transfered to the package euvec.m. *)

(* WARNING: The procedure pscomplement, and the depending  of this \
procedures,
e.g. circle2spv,plotcircle2spv, do not always depend continuously of their
vector aeguments. *)

(* ATTENTION! Most of the functions defined in this package are usable for the 
3-dimensional Moebius space, i.i. the 3-sphere, only! Therefore it should be dim 
= 5, ind = 1. *)

BeginPackage["mcirc`", {"vectorcalc`", "euvec`", "neuvec`","mspher`"}] 
  

control::usage = "control[v,w, prod] measures the orthonormality of the 
spacelike vectors v,w with respect  to the  scalar product prod; 
the default prod is the pseudo-Euclidean scalar product pssp."
  
pscomplement::usage = "pscomplement[vlist] yields a 
pseudo-orthonormal basis of the orthogonal subspace to span[v], the subspace  
spanned by the vector sequence v of a pseudo-Euclidean space with scalar product pssp, the first vectors 
timelike or isotropic, if there are such. pscomplement[v,w] yields pscomplement[{v,w}],
or an error message if span[v,w] is not an Euclidean subspace. pscomplement cannot give reliable results
 for vectors depending on parameters."

circle2spv::usage = "circle2spv[v,w][t]is a parameter  representation of
the circle (or line) in the Euclidean 3-space corresponding to the Euclidean  
2-subspace, spanned by the spacelike vectors v, w of the pseudo-Euclidean 
5-space, ind = 1."

threepoints::usage="threepoints[v,w] yields three points of \
circle2spv[v,w][t]."

adaptsplframe::usage="adaptsplframe[v,w] transforms the arbitrary basis v,w 
of an Euclidean 2-space into a frame adapted to the circle defined by the 
2-space; 
the adapted frame is an orthonormal basis such that the first vector defines 
the plane 
of the circle and the second the sphere with the same radius and center as the 
circle. 
For a line any  orthonormal basis is considered to be adapted."

posvec::usage="posvec[v,w] yields the position vector of the circle \
circle2spv[v,w][t]."  

center::usage="center[v,w] yields the center of the circle \
circle2spv[v,w][t]."  

radius::usage="radius[v,w] yields the radius of the circle \
circle2spv[v,w][t]."  

plotcircle2spv::usage = "plotcircle2spv[v,w, opts] plots the circle (or line) with 
parameter representation circle2spv[v, w][t] with the options opts of 
ParametricPlot3D."
  
circlefamily::usage = "circlefamily[v,w,t,a,b, d:20, opt] plots d elements of \
the family of circles defined by the family ({v[t], w[t]}, a<=t<=b, 
of orthonormal spacelike vectors in the pseudo-Euclidean 5-dimenensional 
vector space of index 1. d = 20 is the default value; options opt are 
transferred to ParametricPlot3D."

circleplane::usage = "circleplane[x,y,z,u,v] yields the spacelike unit vector \
corresponding to the plane in the Euclidean 3-space containing the circle \
with center {x,y,z} and position vector unit[2][{u,v}]; it does not depend on 
the 
radius of the circle."

circlespacevectors::usage = "circlespacevectors[x,y,z,u,v,r] yields the \
adapted 
orthonormal 2-frame of the Euclidean subspace of the pseudo-Euclidean 5-space \
defining the circle with center {x,y,z}, position vector unit[2][{u,v}], and 
radius r; the first vector corresponds to the plane of the circle and the \
second 
to the sphere with the same center and radius as the circle."
 
vspace::usage = "If p is a List of points of the Euclidean n-space, then vspace[p] yields an 
orthonormal basis of the Euclidean subspace in the (n+2)-dimensional Lorentz space being the orthogonal complement 
of the  pseudo-orthogonal subspace corresponding to the maximal subsphere through the points of the List p.
vspace[p1,p2,p3] yields a basis of the Euclidean subspace corresponding to the circle through the points p1,p2,p3."
  
pp::usage = "pp: If V and W are the Euclidean vector subspaces spanned by the 
orthonormal bases {v1, v2}, {w1, w2}  respectively, then pp[v1, v2, w1,w2] 
is the matrix of the composition p = p2 p1 of the orthogonal projections 
p1: V -> W, and p2: W -> V, with respect to the base {v1, v2}."
  
ppdet::usage = "ppdet is the determinant of the matrix pp."
  
pptr::usage = "pptr is the trace of the matrix pp."
  

Begin["`Private`"]
  
 dimm = Global`dim;

indd = Global`ind;
  

 (*Control of orthonormality*)
  
  
control[v_List, w_List, prod_:pssp] := 
    Simplify[(prod[v, v] - 1)^2 + prod[v, w]^2 + (prod[w, w] - 1)^2]

  


(*Now we introduce a parameter representation of the circle  corresponding to \
an Euclidean 2-subspace of the pseudo-Euclidean 5-space, ind = 1. 
First we define the submodule pscomplement which may be useful at other \
opportunities, too.*)

(* pscomplement[v_List,w_List]:= Module[{j=0,w1,w2,w3,chj,d},
   d = pssp[v,v]*pssp[w,w] -pssp[v,w]^2;
If[d<= 0, Print["The vectors don't span a Euclidean 2-space!"]; 
  Return[]];
   Label[chj];
    If[j==dimm, 
      Print["The complement not found. 
      Vectors do not span an Euclidean 2-space?"];
      Return[];Break[],j=j+1];              
     w1 = Simplify[pscross[{v,w,stb[j],stb[1]}]];
    If[w1 == null[5], Goto[chj], w2 = Simplify[pscross[{v,w,w1,stb[1]}]], 
      w2 = Simplify[pscross[{v,w,w1,stb[1]}]]];
    w3 = Simplify[pscross[{v,w,w1,w2}]];
    chsort[{w1/Sqrt[pssp[w1,w1]],w2/Sqrt[pssp[w2,w2]],
        w3/Sqrt[-pssp[w3,w3]]}]]*)
        
pscomplement[v_List]:=chsort[orthonorm[NullSpace[Table[dual[v[[i]]],{i,1,Length[v]}]]]];

pscomplement[v_List,w_List]:=Module[{d},
d = pssp[v,v]*pssp[w,w] -pssp[v,w]^2;
If[d<= 0, Print["The vectors don't span an Euclidean 2-space!"]; 
  Return[]];
pscomplement[{v,w}]];


(* Three points *)

threepoints[v_,w_]:= Module[{b=pscomplement[v,w]},
If[b == Null, Return[];Break[]];
		{stepro[b[[3]]+  b[[1]]], stepro[b[[3]] - b[[1]]], stepro[b[[1]]+ b[[2]]]}]

(* Adapting frames. *)
adaptsplframe[vx_List,vy_List] := Module[{aob,d1,d2,q = 0,v,ortho,nor},
    If[Simplify[pssp[vx, vx]*pssp[vy, vy]-pssp[vx, vy]^2<= 0],
      Print["The span is not 2-dimensional Euclidean!"];Return[],Goto[ortho],
	 Print["May be the span is not 2-dimensional Euclidean!"];Return[]];
	Label[ortho];
	aob = orthonorm[{vx,vy}, normed -> False];
    d1=Simplify[aob[[1]][[5]] - aob[[1]][[1]]];
    d2= Simplify[aob[[2]][[5]] - aob[[2]][[1]]];
    If[d1== 0, q=1;Goto[nor]];
    If[d2==0,v=aob[[1]]; aob[[1]] = aob[[2]];aob[[2]]= v;q=1];
    Label[nor];
    aob[[1]] = aob[[1]]/Sqrt[pssp[aob[[1]], aob[[1]]]];
    aob[[2]] = aob[[2]]/Sqrt[pssp[aob[[2]], aob[[2]]]];
    If[q==1,Return[Simplify[aob]]];
    d1=Simplify[aob[[1]][[5]] - aob[[1]][[1]]];
    d2= Simplify[aob[[2]][[5]] - aob[[2]][[1]]];
    q = Sqrt[d1^2 + d2^2];
    v = aob[[1]]; 
    aob[[1]] = Simplify[ -v*d2/q + aob[[2]]*d1/q];
    aob[[2]] =Simplify[ v*d1/q + aob[[2]]*d2/q];
    Return[aob]];

(*Position vector, center, and radius of the circle defined by an Euclidean \
2-space spanned by the vectors v,w. *)

posvec[vx_List,vy_List] := Module[{ab},
ab=adaptsplframe[vx,vy];
    If[ab[[2]][[1]] == ab[[2]][[5]], Return[{0,0,0}]];
    Return[Take[ab[[1]],{2,4}]]];

center[vx_List,vy_List] := Module[{ab},
ab=adaptsplframe[vx,vy];
    If[ab[[2]][[1]] == ab[[2]][[5]], 
      Return[{Infinity,Infinity,Infinity}]];
    Return[Take[ab[[2]],{2,4}]/(ab[[2]][[1]] - ab[[2]][[5]])]];
    
radius[vx_List,vy_List] := Module[{ab},
ab=adaptsplframe[vx,vy];
    If[ab[[2]][[1]] == ab[[2]][[5]], Return[Infinity]];
    Return[1/(ab[[2]][[5]] - ab[[2]][[1]])]];

(*Plotting circles.*)
	
circle2spv[v_List,w_List][t_]:= Module[{psc},
    psc = pscomplement[v,w];
    If[psc == Null, Return[]];
      stepro[-psc[[2]]*Cos[t] -psc[[3]]*Sin[t] + psc[[1]]]];
      
circle2spv[v_?MatrixQ][t_] := circle2spv[v[[1]],v[[2]]][t]; 

 plotcircle2spv[v_?MatrixQ,opt___]:= plotcircle2spv[v[[1]],v[[2]],opt];
     
plotcircle2spv[v_,w_,opt___]:= 
  ParametricPlot3D[circle2spv[v,w][t], {t,0,2*Pi}, opt];
 


      circlefamily[v_,w_,t_,a_,b_, d_:20, opt___] := 
      Table[plotcircle2spv[v[t],w[t],opt],{t, a, b, (b-a)/d}]

(*Finding the Euclidean 2-space of the circle through three points.*)

(* WARNING: Replacing pscross[{...}] by psCross[...] may lead to very strange
errors if applied to floating point vectors; try it with *)

(* zz[u_]:= vspace[{1,0,u},{0,1,u},{-1,0,u}] *)

(* and compare the results for u = 3, u = 3.0, or u = 5, u = 5.0 ! REASON? *)

circleplane[x_,y_,z_,u_,v_] :={x Cos[u] Cos[v]+y Cos[v] Sin[u]+z Sin[v],
Cos[u] Cos[v],Cos[v] Sin[u],Sin[v],x Cos[u] Cos[v]+y Cos[v] Sin[u]+z Sin[v]}
    
circlespacevectors[x_,y_,z_,u_,v_,r_] :={circleplane[x,y,z,u,v],
    spherevec[x,y,z,r]}

vspace[p_] := Module[{q, vn},If[indd!= 1, Print["Warning: The index is not 1 !"]];
If[Not[MatrixQ[p]], Print["Wrong argument!"];Return[]];
q = Map[Composition[ dual,invstepro],p]; 
vn = orthonorm[NullSpace[q]];chsort[vn]]

vspace[p1_,p2_,p3_]:= vspace[{p1,p2,p3}];

  
(*Invariants of pairs of circles.*)
  
  
 pp[v1_, v2_, w1_, w2_] := 
    {{pssp[v1, w1]^2 + pssp[v1, w2]^2, 
      pssp[v1, w1]*pssp[v2, w1] + pssp[v1, w2]*pssp[v2, w2]}, 
     {pssp[v1, w1]*pssp[v2, w1] + pssp[v1, w2]*pssp[v2, w2], 
      pssp[v2, w1]^2 + pssp[v2, w2]^2}}
  
  
ppdet[v1_, v2_, w1_, w2_] := Det[pp[v1, v2, w1, w2]]
  
  
pptr[v1_, v2_, w1_, w2_] := Sum[pp[v1, v2, w1, w2][[i,i]], {i, 1, 2}]

End[]
  
  
Protect[adaptsplframe, center, circle2spv,
circlefamily, \
circleplane, 
circlespacevectors, control, 
pp, ppdet, pptr, plotcircle2spv,
 posvec, pscomplement, radius, \
threepoints, vspace]

EndPackage[]

