(* ::Package:: *)

(* mspher.m *)

(* Version 2 *)

(* Created by Rolf Sulanke *)

(* Started March 20, 1995 , revised Sep 24,1998, last revision February 25,
 April 14, 2004.Reviused again 2012,July 26 *)

(*This package contains procedures and functions for working in the Moebius
 geometry of spheres. It continues the package euvec.m which contains concepts of the
 Euclidean geometry of spheres. Operations and functions of pseudo-Euclidean vector algebra
 (see the package neuvec.m and the notebook pseuklid.nb) are used. All definitions assume 
that there is given a fixed pseudo-orthonormal coordinate system of index 1 and given dimension.
Definitions usiong isotropic-orthonormal coordinates are given in the package moeb2.m. 
For applications and further  procedures see the notebook mspheres.nb.*)

(* Free software: Feel free to apply, distribute or change the packages and
notebooks of the item "Spheres". Please, cite the source and notice the
changes, or new concepts,  which you introduced.*)

(*Remark: The functions plotsphere, psphere, paramsphere,showsphere are deleted now; replace
 them in an appropriate way by euklidsphere, euklidsphereplot3D. 
New functions: vradius,vcenter.*)
 
(* Changes. 2003, Oct: spacevec renamed to spherevec. confinv renmed to
 coxeterinv. 
 planev renamed to planevec. spunit is generalized; the more special old version
 is preserved. 
 April 2004: hspherevector introduced. spherevec may be replaced by the 
 more general hspherevector; it is obsolete now *)

BeginPackage["mspher`",{"neuvec`","euvec`","vectorcalc`"}]     



spt::usage ="spt[v] projects the vector v onto the vector
v/v[[1]]-stb[1] in the (dim-1)-space orthogonal to stb[1].In particular, it maps the 
isotropic vector u onto the ncorresponding point of the unit hypersphere. 
Warning: The index must equal 1, and the result is valid for isotropic vectors in 
pseudo-orthogonal coordinates only!";

stepro::usage ="stepro is the stereographic projection in  pseudo-Euclidean
terms (ind = 1), with values in the Euclidean (dim - 2)-space.Warning: the 
vector v must be given in pseudo-orthonormal coordinates, and
stepro[v] is correct only for isotropic vectors v!";

invstepro::usage="invstepro is the inverse of the stereographic projection
 stepro (ind = 1).";


spunit::usage = "spunit[n][sList] is a parameter representation of the manifold
of all spacelike unit vectors in the n+1-dimensional pseudo-Euclidean space of
index 1 in pseudo-orthonormal coordinates; the ranges are: 0<=s[[1]]<=2Pi, -Pi/2<=s[[j]]<=Pi/2 (1<j<n), s[[n]]
 arbitrary 
real; spunit[s,t,u,v]= spunit[4][{s,t,u,v}]."

pstg5frame::usage = "pstg5frame[s,t,u,v] is a pseudo-orthonormal moving frame
adapted to the manifold spunit[4][{s,t,u,v}] of spacelike unit vectors in 
pseudo-orthonormal coordinates; the last vector equals spunit[4][{s,t,u,v}]." 

vradius::usage = "vradius[vx] is the radius of the hypersphere in the Euclidean
d-space corresponding to the spacelike vector vx in pseudo-orthonormal coordinates 
of the pseudo-Euclidean (d+2)-space of index 1, and Infinity if this sphere is a plane."

vcenter::usage = "vcenter[vx] is the center of the hypersphere in the Euclidean 
d-space corresponding to the spacelike vector vx of the pseudo-Euclidean
 (d+2)-space of index 1 in pseudo-orthonormal coordinates, and the infinite point 
if this hypersphere is a plane."

euklidsphere::usage = "euklidsphere[vx][u,v] is a parameter representation
of the sphere (resp. plane), with geographical parameters u,v, in the Euclidean
3-space, which corresponds to the spacelike vector vx in pseudo-orthonormal coordinates." 

euklidsphereplot3D::usage = "euklidsphereplot3D[vx,u0:-1,u1:1,v0:-1,v1:1,opts] 
is the plot of a region of the sphere or the plane corresponding to the spacelike vector 
vx in pseudo-orthonormal coordinates; the region is defined by the parameters u0, u1, v0, v1. With 
the default values the whole sphere or a square with side length 2 of the plane 
is plotted; options opts are transferred to ParametricPlot3D."

sph4ptsvec::usage = "sph4ptsvec[p1, p2, p3, p4] yields a spacelike vector of the
 pseudo-Euclidean 5-space  in pseudo-orthonormal coordinates corresponding to the sphere 
through the four points p1, p2, p3, p4 (in general position) of the Euclidean 3-space.";

hspherevector::usage = "hspherevector[m, r] yields a spacelike unit vector in pseudo-orthonormal 
coordinates of the pseudo-Euclidean (d+2)-space corresponding to the sphere with center m and
radius r of the d-dimensional Euclidean space.";

spherevec::usage = "spherevec[x, y, z, r] yields a spacelike unit vector in pseudo-orthonormal coordinates 
of the pseudo-Euclidean 5-space corresponding to the sphere with center {x, y, z} and radius
r of the Euclidean 3-space.";

gencos::usage = "gencos[v,w] is the generalized cos of the (may be imginary) 
intersection angle of the spheres defined by the spacelike vectors v,w.";

coxeterinv::usage = "coxeterinv[r1, r2, d] is the conformal invariant of two
hyperspheres with radii r1, r2 and distance d of their centers.";

planevec::usage = "planevec[a,b,c,p] yields a spacelike vector  in pseudo-orthonormal coordinates
of the pseudo-Euclidean  5-space of index 1 defining the plane of the Euclidean 3-space with
equation ax+by+cz=p.";

hplanevec::usage = "If d = Length[w}, hplanevec[wList,p] yields a spacelike vector 
of the pseudo-Euclidean  (d+2)-space of index 1 in pseudo-orthonormal coordinates
 defining the hyperplane of the Euclidean d-space with equation w.vx=p.";

plane::usage = "plane[w][u,v] is a parameter representation of the plane in the Euclidean 
3-space corresponding to the spacelike vector w with w[[1]] = w[[5]] in pseudo-orthonormal coordinates."




Begin["`Private`"]

dimm = Global`dim;

indd = Global`ind; 


(*Projections of isotropic vectors on sphere points*)



spt[u_List] := Take[u/u[[1]],-Length[u]+1];
				
stepro[v_List] := sterproj[spt[v]]

(*The inverse  projection: sphere points - isotropic vector*)


invstepro[v_List] := Simplify[Prepend[invsterproj[v],1]]


(* Definition of the pseudosphere  in the pseudo-Euclidean n+1-Space: *)

spunit[n_][s_List] := Module[{ss,uu},
	If[Length[s] != n, Print["Incorrect arguments!"]; Return[]];
	ss = Take[s, n-1];
	uu = unit[n-1][ss];
	PrependTo[uu, 0];
	Return[uu*Cosh[s[[n]]] + stb[1,n+1]*Sinh[s[[n]]]]]

(* Case n = 4; the corresponding spheres in the Euclidean 3-space *)

spunit[s_,t_,u_,v_]:=
{Sinh[v], Cos[s] Cos[t] Cos[u] Cosh[v],Cos[t] Cos[u] Cosh[v] Sin[s],
  Cos[u] Cosh[v] Sin[t],Cosh[v] Sin[u]}
	
pstg5frame[s_, t_, u_, v_] :=
{{Cosh[v],Cos[s] Cos[t] Cos[u] Sinh[v],Cos[t] Cos[u] Sin[s] Sinh[v],
    Cos[u] Sin[t] Sinh[v],Sin[u] Sinh[v]},{0,-Sin[s],Cos[s],0,
    0},{0,-Cos[s] Sin[t],-Sin[s] Sin[t],Cos[t],
    0},{0,-Cos[s] Cos[t] Sin[u],-Cos[t] Sin[s] Sin[u],-Sin[t] Sin[u],
    Cos[u]},{Sinh[v],Cos[s] Cos[t] Cos[u] Cosh[v],
    Cos[t] Cos[u] Cosh[v] Sin[s],Cos[u] Cosh[v] Sin[t],Cosh[v] Sin[u]}}


(*	{{- Sin[s],Cos[s],0,0,0},
		{-Cos[s] Sin[t],- Sin[s] Sin[t],Cos[t],0,0},
		{-Cos[s] Cos[t]  Sin[u],-Cos[t]  Sin[s] Sin[u],- Sin[t] Sin[u],Cos[u],0},
		{Cos[s] Cos[t] Cos[u] Sinh[v],Cos[t] Cos[u] Sin[s] Sinh[v],
      Cos[u] Sin[t] Sinh[v],Sin[u] Sinh[v],Cosh[v]},
		{Cos[s] Cos[t] Cos[u] Cosh[v],Cos[t] Cos[u] Cosh[v] Sin[s],
      Cos[u] Cosh[v] Sin[t],Cosh[v] Sin[u],Sinh[v]}} *)

vradius[wx_List] := Module[{s,q, vx,d= Length[wx]},
      q = Simplify[pssp[wx,wx]];
      If[q<= 0,Print[wx, " is not spacelike!"];Return[]];
      vx = wx/Sqrt[q];
      s=Simplify[vx[[1]]- vx[[d]]];
      If[s === 0, Return[Infinity],Return[1/s],Return[1/s]]]

vcenter[wx_List] := Module[{s,d = Length[wx], q},
	q = pssp[wx,wx];
  If[q<= 0,Print[wx, " is not spacelike!"];Return[]];
  s=Simplify[wx[[1]]- wx[[d]]];
  If[s === 0, Return[Infinity]];Return[Take[wx,{2,d - 1}]/s]]
     

euklidsphere[va_List][u_,v_] := Module[{q},
      q = pssp[va,va];
      If[q<= 0, Print[va, " is not spacelike!"]; Return[]];
      w=va/Sqrt[q];
      If[w[[1]] == w[[5]], Goto[pla]];
      Return[Simplify[sphere[vcenter[w],vradius[w]][u*Pi,v*Pi/2]]];
      Label[pla];
       p = w[[1]];
      w = Take[w,{2,4}];
      ob = orthocomplement[{w}];
        Return[w*p + ob[[1]]*u + ob[[2]]*v]
      ]/;Length[va]== 5
    
euklidsphereplot3D[vn_List,u0_:-1,u1_:1, v0_:-1,v1_:1,opts___]:=
  ParametricPlot3D[Evaluate[euklidsphere[vn][u,v]],{u,u0,u1}, {v,v0,v1},opts
  ]/;Length[vn]== 5
    
(*Definition of a spacelike vector defining the sphere through four points of
 the 3e-space*)

sph4ptsvec[p1_List,p2_List,p3_List, p4_List] := Module[{v=null[5]},
		If[Length[p1]!=3||Length[p2]!=3||Length[p3]!=3||Length[p4]!=3,
		Print["An argument does not have the correct
dimension!"];Return[]];
v=Simplify[ 
        pscross[{invstepro[p1], invstepro[p2],invstepro[p3],invstepro[p4]}]];
		If[v===null[5],Print["The points are not in general
position!"];Return[],  Return[v], Return[v]]]

(*Definition of a spacelike vector defining the hypersphere with given center and
 radius *)
 
 hspherevector[m_List,r_] := Module[{a,vm, d = Length[m]}, 
    a = m.m;
    vm =Join[{0}, m, {0}];
    Return[
      (2*vm - stb[d+2,d+2]*(1 + r^2 - a) +
             stb[1,d+2]*(1 - r^2 + a))/(2*r)]];
             
(*spherevec is obsolete now *)             

spherevec[x_, y_, z_, r_] := {1 - r^2+ x^2 + y^2 + z^2, 2 x,  2 y, 
2 z, -1 - r^2 + x^2 + y^2 + z^2 }/(2 r) 

(*Definition of a spacelike vector defining the plane with equation ax+by+cz=p
 of the 3e-space*)

planevec[a_,b_,c_, p_] := If[Not[a^2+b^2+c^2===0],
	Return[{p, a, b, c, p}],Print["Normal vector null is not admissible!"];
    Return[]]

(*The generalization to higher dimensions is obvious: *)

hplanevec[w_List,p_] := If[Not[w.w===0],vw = Flatten[{{p},w,{p}}];
	Return[vw],Print["Normal vector null is not admissible!"];Return[]]

(* The parameter representation of a plane in the Euclidean 3-space. *)
	
plane[vc_List][u_,v_] := Module[{q,p,w,ob},
    q=pssp[vc,vc];
    If[Length[vc] != 5 || q<=0|| 
        vc[[1]]!= vc[[5]],Print[vc, " incorrect argument!"];
      Return[]]; 
    w=vc/Sqrt[q];
    p = w[[5]];
    w = Take[w,{2,4}];
    ob = orthocomplement[{w}];
    Return[Simplify[w*p + ob[[1]]*u + ob[[2]]*v]]]

(*Invariants of a pair of spheres*)

gencos[v_List, w_List] := Simplify[pssp[v, w]/Sqrt[pssp[v,v] pssp[w,w]]]

coxeterinv[r1_, r2_, d_] := (r1^2 + r2^2 -d^2)/(2 r1 r2)

End[]

Protect[spt, stepro, invstepro, spunit,euklidsphere,
 euklidsphereplot3D, pstg5frame, sph4ptsvec, spherevec, gencos, coxeterinv, 
plane, planevec, hplanevec, vradius, vcenter,hspherevector]

EndPackage[]
